@@ -166,7 +166,7 @@ public :
         return true ;
     }
 
-    /// \brief truncate a point that it stays inside the domain
+    /// \brief truncate a point so that it stays inside the domain
     /// \param p_point  point to truncate
     void truncatePoint(Eigen::ArrayXd &p_point) const
     {
@@ -188,6 +188,20 @@ public :
     {
         return (p_nbPoints == (m_nbStep + 1).prod());
     }
+
+    /// \brief Get back 1D grids in each dimension
+  std::vector< std::shared_ptr<Eigen::ArrayXd> > get1DGrids() const
+  {
+    std::vector< std::shared_ptr<Eigen::ArrayXd> > retGrids(m_lowValues.size());
+    for (int id =0; id < m_lowValues.size(); ++id)
+      {
+	std::shared_ptr<Eigen::ArrayXd> grid1D= std::make_shared<Eigen::ArrayXd>(m_nbStep(id)+1);
+	for (int j=0; j< m_nbStep(id)+1; ++j)
+	  (*grid1D)(j)= m_lowValues(id)+ m_step(id)*j;
+	retGrids[id]= grid1D;
+      }
+    return retGrids;
+  }
 };
 
 }
