@@ -19,9 +19,11 @@ namespace StOpt
 /// \param p_grid   global grid of the problem
 /// \param p_values local values associated to the processor
 /// \param p_bdimToSplit    Dimensions to split for parallelism
+/// \param  p_world         MPI communicator
 Eigen::ArrayXd  reconstructProc0ForIntMpi(const Eigen::ArrayXi &p_point,
         const std::shared_ptr<StOpt::RegularSpaceIntGrid> &p_grid,
         const std::shared_ptr< Eigen::ArrayXXd >    &p_values,
-        const Eigen::Array< bool, Eigen::Dynamic, 1>   &p_bdimToSplit);
+        const Eigen::Array< bool, Eigen::Dynamic, 1>   &p_bdimToSplit,
+        const boost::mpi::communicator &p_world);
 }
 #endif /* RECONSTRUCTPROC0FORINTMPI_H */
