@@ -81,11 +81,7 @@ void recursiveExploration1DNoBound(Eigen::ArrayXc &p_levelCurrent,
         return ;
 
     // achieve 1D Hierarchization for the current node and given direction
-#ifdef _MSC_VER  // VS Bug ?
-    HierDehier().operator()<T, TT>(p_levelCurrent, p_positionCurrent, p_iterLevel, p_idim, p_dataSet, p_source, p_output);
-#else
     HierDehier().template operator()<T, TT>(p_levelCurrent, p_positionCurrent, p_iterLevel, p_idim, p_dataSet, p_source, p_output);
-#endif
 
     // recursive if current is  not a leaf
     for (size_t idd = 0 ; idd < p_idimRemain; ++idd)
