@@ -31,6 +31,8 @@ public :
 /// \brief main constructor
     StateWithStocks(const int &p_regime, const Eigen::ArrayXd &p_ptStock, const Eigen::ArrayXd &p_stochasticRealisation):
       m_ptStock(p_ptStock), m_stochasticRealisation(p_stochasticRealisation), m_regime(p_regime) ,m_iSimul(-1) {}
+  StateWithStocks(const int &p_regime, const Eigen::ArrayXd &p_ptStock, const Eigen::ArrayXd &p_stochasticRealisation, const int & p_iSimul):
+      m_ptStock(p_ptStock), m_stochasticRealisation(p_stochasticRealisation), m_regime(p_regime) ,m_iSimul(p_iSimul) {}
 
 /// \brief accessor
 ///@{
