@@ -55,7 +55,11 @@ public :
 #endif
                              );
 
-    /// \brief Define one step arbitraging between possible commands
+    /// \brief default
+    SimulateStepRegressionCut() {}
+    virtual ~SimulateStepRegressionCut() {}
+
+  /// \brief Define one step arbitraging between possible commands
     /// \param p_statevector    Vector of states (regime, stock descritor, uncertainty)
     /// \param p_phiInOut       actual contract values modified at current time step by applying an optimal command (number of function by numver of simulations)
     void oneStep(std::vector<StateWithStocks > &p_statevector, Eigen::ArrayXXd  &p_phiInOut) const ;
