@@ -2,6 +2,9 @@
 // All Rights Reserved
 // This code is published under the GNU Lesser General Public License (GNU LGPL)
 #ifdef USE_MPI
+#ifdef _OPENMP
+#include <omp.h>
+#endif
 #include "geners/Reference.hh"
 #include "geners/vectorIO.hh"
 #include "StOpt/dp/SimulateStepRegressionCutDist.h"
