@@ -52,7 +52,11 @@ public :
                                   const  std::shared_ptr<OptimizerDPCutBase > &p_pOptimize,
                                   const bool &p_bOneFile, const boost::mpi::communicator &p_world);
 
-    /// \brief Define one step arbitraging between possible commands
+   /// \brief default
+    SimulateStepRegressionCutDist() {}
+    virtual ~SimulateStepRegressionCutDist() {}
+
+  /// \brief Define one step arbitraging between possible commands
     /// \param p_statevector    Vector of states (regime, stock descriptor, uncertainty)
     /// \param p_phiInOut       actual contract value modified at current time step by applying an optimal command (size number of functions by number of simulations)
     void oneStep(std::vector<StateWithStocks > &p_statevector, Eigen::ArrayXXd  &p_phiInOut) const;
