@@ -48,7 +48,7 @@ public :
 
     /// \brief Constructor
     TransitionStepRegressionDP(const  std::shared_ptr<FullGrid> &p_pGridCurrent,
-                               const  std::shared_ptr<FullGrid> &p_pridPrevious,
+                               const  std::shared_ptr<FullGrid> &p_pGridPrevious,
                                const  std::shared_ptr<OptimizerDPBase > &p_pOptimize
 #ifdef USE_MPI
                                , const boost::mpi::communicator &p_world
@@ -80,7 +80,6 @@ public :
     /// \param p_name                 name used for object
     /// \param p_iStep                 Step number or identifier for time step
     /// \param p_phiIn                for each regime the function value ( nb simulation ,nb stocks)
-    /// \param p_control              Optimal control ( nb simulation ,nb stocks) for each control
     /// \param p_condExp               conditional expectation operator
     void dumpBellmanValues(std::shared_ptr<gs::BinaryFileArchive> p_ar, const std::string &p_name, const int &p_iStep, const std::vector< std::shared_ptr< Eigen::ArrayXXd > > &p_phiIn,
                            const  std::shared_ptr<BaseRegression>    &p_condExp) const;
