@@ -22,6 +22,7 @@
 #include "StOpt/dp/OptimizerDPBase.h"
 #include "StOpt/dp/OptimizerDPTreeBase.h"
 #include "StOpt/dp/OptimizerSwitchBase.h"
+#include "StOpt/dp/OptimizerDPCutBase.h"
 #include "StOpt/core/grids/SpaceGrid.h"
 #include "StOpt/core/grids/FullGrid.h"
 #include "StOpt/core/grids/RegularSpaceIntGrid.h"
@@ -34,10 +35,12 @@
 #include "StOpt/dp/FinalStepDP.h"
 #include "StOpt/dp/TransitionStepRegressionDP.h"
 #include "StOpt/dp/TransitionStepRegressionSwitch.h"
+#include "StOpt/dp/TransitionStepRegressionDPCut.h"
+#include "StOpt/dp/TransitionStepTreeDP.h"
 #include "StOpt/dp/SimulateStepRegression.h"
 #include "StOpt/dp/SimulateStepRegressionControl.h"
+#include "StOpt/dp/SimulateStepRegressionCut.h"
 #include "StOpt/dp/SimulateStepSwitch.h"
-#include "StOpt/dp/TransitionStepTreeDP.h"
 #include "StOpt/dp/SimulateStepTree.h"
 #include "StOpt/dp/SimulateStepTreeControl.h"
 #include "StOpt/semilagrangien/SemiLagrangEspCond.h"
@@ -46,9 +49,11 @@
 #include "StOpt/dp/FinalStepZeroDist.h"
 #include "StOpt/dp/TransitionStepRegressionDPDist.h"
 #include "StOpt/dp/TransitionStepRegressionSwitchDist.h"
+#include "StOpt/dp/TransitionStepTreeDPDist.h"
+#include "StOpt/dp/TransitionStepRegressionDPCutDist.h"
 #include "StOpt/dp/SimulateStepRegressionDist.h"
 #include "StOpt/dp/SimulateStepRegressionControlDist.h"
-#include "StOpt/dp/TransitionStepTreeDPDist.h"
+#include "StOpt/dp/SimulateStepRegressionCutDist.h"
 #include "StOpt/dp/SimulateStepTreeDist.h"
 #include "StOpt/dp/SimulateStepTreeControlDist.h"
 #include "StOpt/core/parallelism/reconstructProc0Mpi.h"
@@ -280,6 +285,57 @@ class  PyOptimizerDPBase :  public StOpt::OptimizerDPBase
     }
 };
 
+/// wrapper for optimization object for Optimization with cut in the concave or convex case
+/// 
+class  PyOptimizerDPCutBase : public StOpt::OptimizerDPCutBase
+{
+      using StOpt::OptimizerDPCutBase::OptimizerDPCutBase;
+
+    std::vector< std::array<double, 2 > > getCone(const  std::vector<  std::array< double, 2>  > &) const override
+    {
+        using retCone = std::vector< std::array< double, 2> >;
+        PYBIND11_OVERLOAD_PURE(retCone, StOpt::OptimizerDPCutBase, getCone);
+    }
+  
+    Eigen::Array< bool, Eigen::Dynamic, 1>   getDimensionToSplit() const override
+    {
+        using boolArray = Eigen::Array< bool, Eigen::Dynamic, 1> ;
+        PYBIND11_OVERLOAD_PURE(boolArray, StOpt::OptimizerDPCutBase, getDimensionToSplit);
+    }
+  
+   Eigen::ArrayXXd    stepOptimize(const   std::shared_ptr< StOpt::SpaceGrid> &, const Eigen::ArrayXd   &,  const std::vector< StOpt::ContinuationCuts  > &) const override
+    {
+        PYBIND11_OVERLOAD_PURE(Eigen::ArrayXXd, StOpt::OptimizerDPCutBase, stepOptimize);
+    }
+
+    void stepSimulate(const std::shared_ptr< StOpt::SpaceGrid>   &, const std::vector< StOpt::ContinuationCuts  > &,
+                              StOpt::StateWithStocks &,  Eigen::Ref<Eigen::ArrayXd>) const override
+    {
+        PYBIND11_OVERLOAD_PURE(void, StOpt::OptimizerDPCutBase, stepSimulate) ;
+    }
+  
+    int getNbRegime() const override
+    {
+        PYBIND11_OVERLOAD_PURE(int,  StOpt::OptimizerDPCutBase, getNbRegime);
+    }
+
+
+    std::shared_ptr< StOpt::SimulatorDPBase > getSimulator() const override
+    {
+        //using sharedPtrSim=std::shared_ptr< StOpt::SimulatorSwitchBase >
+        PYBIND11_OVERLOAD_PURE(std::shared_ptr< StOpt::SimulatorDPBase >,  StOpt::OptimizerDPCutBase, getSimulator);
+    }
+
+    int getSimuFuncSize() const  override
+    {
+        PYBIND11_OVERLOAD_PURE(int, StOpt::OptimizerDPCutBase, getSimuFuncSize);
+    }
+      int getNbControl() const  override
+    {
+        PYBIND11_OVERLOAD_PURE(int, StOpt::OptimizerDPCutBase, getNbControl);
+    }
+};
+
 
 ///wrapper for optimization object for Optimization Switching Base object for regression for dynamic programming
 ///**************************************************************************************************
@@ -573,6 +629,75 @@ public :
     }
 };
 
+/// mapping for transition step with regressions and cuts
+class PyTransitionStepRegressionDPCut: StOpt::TransitionStepRegressionDPCut
+{
+public :
+    PyTransitionStepRegressionDPCut(const std::shared_ptr< StOpt::SpaceGrid >   &p_gridCurrent,  const std::shared_ptr< StOpt::SpaceGrid >   &p_gridPrevious, const  std::shared_ptr< StOpt::OptimizerDPCutBase>   &p_optimizer):
+        StOpt::TransitionStepRegressionDPCut(std::dynamic_pointer_cast<StOpt::FullGrid>(p_gridCurrent),
+                                          std::dynamic_pointer_cast<StOpt::FullGrid>(p_gridPrevious),
+                                          p_optimizer
+#ifdef USE_MPI
+                                          , boost::mpi::communicator()
+#endif
+                                         ) {  }
+
+    py::list  oneStepWrap(const std::vector< Eigen::ArrayXXd >  &p_phiIn,
+                          const std::shared_ptr< StOpt::BaseRegression>     &p_condExp) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  input;
+        input.reserve(p_phiIn.size());
+        for (const auto &item : p_phiIn)
+        {
+            input.push_back(std::make_shared<Eigen::ArrayXXd>(item));
+        }
+       std::vector< std::shared_ptr< Eigen::ArrayXXd > > retLoc = StOpt::TransitionStepRegressionDPCut::oneStep(input, p_condExp);
+       py::list res ;
+        for (const auto &item : retLoc)
+            res.append(*item);
+        return res;
+    }
+
+    void dumpContinuationCutsValuesWrap(std::shared_ptr<BinaryFileArchiveStOpt>  p_ar, const std::string &p_name, const int &p_iStep, const  py::list  &p_phiIn,  const   std::shared_ptr<StOpt::BaseRegression>    &p_condExp) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  phiIn = convertFromListShPtr<Eigen::ArrayXXd>(p_phiIn);
+        StOpt::TransitionStepRegressionDPCut::dumpContinuationCutsValues(p_ar, p_name, p_iStep, phiIn,  p_condExp);
+    }
+
+    void dumpBellmanCutsValuesWrap(std::shared_ptr<BinaryFileArchiveStOpt>  p_ar, const std::string &p_name, const int &p_iStep, const  py::list  &p_phiIn,   const   std::shared_ptr<StOpt::BaseRegression>    &p_condExp) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  phiIn = convertFromListShPtr<Eigen::ArrayXXd>(p_phiIn);
+        StOpt::TransitionStepRegressionDPCut::dumpBellmanCutsValues(p_ar, p_name, p_iStep, phiIn,  p_condExp);
+    }
+};
+
+
+// Wrapper for SimulateStepRegressionCut
+class PySimulateStepRegressionCut : StOpt::SimulateStepRegressionCut
+
+{
+public :
+    PySimulateStepRegressionCut(BinaryFileArchiveStOpt &p_ar,  const int &p_iStep,  const std::string &p_nameCont,
+                             const   std::shared_ptr< StOpt::SpaceGrid > &p_pGridFollowing,
+                             const  std::shared_ptr< StOpt::OptimizerDPCutBase > &p_pOptimize):
+        StOpt::SimulateStepRegressionCut(p_ar, p_iStep, p_nameCont, p_pGridFollowing, p_pOptimize
+#ifdef USE_MPI
+                                      , boost::mpi::communicator()
+#endif
+                                     )
+    {}
+
+    py::list  oneStepWrap(const py::list    &p_statevector,  const Eigen::ArrayXXd   &p_phiInOut) const
+    {
+        std::vector<StOpt::StateWithStocks> stateLoc = convertFromList<StOpt::StateWithStocks>(p_statevector);
+        Eigen::ArrayXXd phiLoc(p_phiInOut);
+        StOpt::SimulateStepRegressionCut::oneStep(stateLoc, phiLoc);
+        py::list ret;
+        ret.append(py::cast(stateLoc));
+        ret.append(py::cast(phiLoc));
+        return ret ;
+    }
+};
 
 /// mapping for transition step with switching
 class PyTransitionStepRegressionSwitch: StOpt::TransitionStepRegressionSwitch
@@ -913,6 +1038,68 @@ public :
 
 };
 
+/// mapping for transition step with regressions and cuts
+class PyTransitionStepRegressionDPCutDist: StOpt::TransitionStepRegressionDPCutDist
+{
+public :
+    PyTransitionStepRegressionDPCutDist(const std::shared_ptr< StOpt::SpaceGrid >   &p_gridCurrent,  const std::shared_ptr< StOpt::SpaceGrid >   &p_gridPrevious, const  std::shared_ptr< StOpt::OptimizerDPCutBase>   &p_optimizer):
+        StOpt::TransitionStepRegressionDPCutDist(std::dynamic_pointer_cast<StOpt::FullGrid>(p_gridCurrent),
+                                          std::dynamic_pointer_cast<StOpt::FullGrid>(p_gridPrevious),
+                                          p_optimizer   , boost::mpi::communicator() ) {  }
+
+    py::list  oneStepWrap(const std::vector< Eigen::ArrayXXd >  &p_phiIn,
+                          const std::shared_ptr< StOpt::BaseRegression>     &p_condExp) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  input;
+        input.reserve(p_phiIn.size());
+        for (const auto &item : p_phiIn)
+        {
+            input.push_back(std::make_shared<Eigen::ArrayXXd>(item));
+        }
+       std::vector< std::shared_ptr< Eigen::ArrayXXd > > retLoc = StOpt::TransitionStepRegressionDPCutDist::oneStep(input, p_condExp);
+       py::list res ;
+        for (const auto &item : retLoc)
+            res.append(*item);
+        return res;
+    }
+
+    void dumpContinuationCutsValuesWrap(std::shared_ptr<BinaryFileArchiveStOpt>  p_ar, const std::string &p_name, const int &p_iStep, const  py::list  &p_phiIn,  const   std::shared_ptr<StOpt::BaseRegression>    &p_condExp, const bool & p_bOneFile) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  phiIn = convertFromListShPtr<Eigen::ArrayXXd>(p_phiIn);
+        StOpt::TransitionStepRegressionDPCutDist::dumpContinuationCutsValues(p_ar, p_name, p_iStep, phiIn,  p_condExp,p_bOneFile);
+    }
+
+    void dumpBellmanCutsValuesWrap(std::shared_ptr<BinaryFileArchiveStOpt>  p_ar, const std::string &p_name, const int &p_iStep, const  py::list  &p_phiIn,   const   std::shared_ptr<StOpt::BaseRegression>    &p_condExp, const bool & p_bOneFile) const
+    {
+        std::vector< std::shared_ptr< Eigen::ArrayXXd > >  phiIn = convertFromListShPtr<Eigen::ArrayXXd>(p_phiIn);
+        StOpt::TransitionStepRegressionDPCutDist::dumpBellmanCutsValues(p_ar, p_name, p_iStep, phiIn,  p_condExp,p_bOneFile);
+    }
+};
+
+
+// Wrapper for SimulateStepRegressionCutDist
+class PySimulateStepRegressionCutDist : StOpt::SimulateStepRegressionCutDist
+
+{
+public :
+    PySimulateStepRegressionCutDist(BinaryFileArchiveStOpt &p_ar,  const int &p_iStep,  const std::string &p_nameCont,
+				    const   std::shared_ptr< StOpt::SpaceGrid > &p_pGridFollowing,
+				    const  std::shared_ptr< StOpt::OptimizerDPCutBase > &p_pOptimize,
+				    const bool &p_bOneFile):
+      StOpt::SimulateStepRegressionCutDist(p_ar, p_iStep, p_nameCont,std::dynamic_pointer_cast<StOpt::FullGrid>( p_pGridFollowing), p_pOptimize,
+					     p_bOneFile, boost::mpi::communicator()){}
+
+    py::list  oneStepWrap(const py::list    &p_statevector,  const Eigen::ArrayXXd   &p_phiInOut) const
+    {
+        std::vector<StOpt::StateWithStocks> stateLoc = convertFromList<StOpt::StateWithStocks>(p_statevector);
+        Eigen::ArrayXXd phiLoc(p_phiInOut);
+        StOpt::SimulateStepRegressionCutDist::oneStep(stateLoc, phiLoc);
+        py::list ret;
+        ret.append(py::cast(stateLoc));
+        ret.append(py::cast(phiLoc));
+        return ret ;
+    }
+};
 
 
 Eigen::ArrayXd  pyReconstructProc0Mpi(const Eigen::ArrayXd &p_point, const std::shared_ptr< StOpt::SpaceGrid> &p_grid, const Eigen::ArrayXXd    &p_values, const Eigen::Array< bool, Eigen::Dynamic, 1>   &p_bdimToSplit)
@@ -1067,6 +1254,39 @@ PYBIND11_MODULE(StOptGlobal, m)
     .def("getSimuFuncSize", &StOpt::OptimizerDPBase::getSimuFuncSize)
     ;
 
+    // map optimizer DP with cuts for concave optimization
+    //*****************************************************
+    py::class_<StOpt::OptimizerDPCutBase, std::shared_ptr<StOpt::OptimizerDPCutBase>, PyOptimizerDPCutBase>(m, "OptimizerDPCutBase")
+    .def("getCone",  [](const StOpt::OptimizerDPCutBase & p_x, const  std::vector< Eigen::ArrayXd >    &p_region)
+    {
+        std::vector< std::array< double, 2> > region;
+        region.reserve(p_region.size());
+        for (const auto &item : p_region)
+        {
+            region.push_back(std::array< double, 2> {{ item[0], item[1]}});
+        }
+        std::vector< std::array< double, 2> > resLoc = p_x.getCone(region);
+        std::vector< Eigen::ArrayXd > result;
+        result.reserve(resLoc.size());
+        for (const auto &item : resLoc)
+        {
+            Eigen::ArrayXd dom(2);
+            dom(0) = item[0];
+            dom(1) = item[1];
+            result.push_back(dom);
+        }
+        return result;
+    }
+        )
+    .def("getDimensionToSplit",  &StOpt::OptimizerDPCutBase::getDimensionToSplit)
+    .def("stepOptimize", &StOpt::OptimizerDPCutBase::stepOptimize) 
+    .def("stepSimulate", &StOpt::OptimizerDPCutBase::stepSimulate)
+    .def("getNbRegime", &StOpt::OptimizerDPCutBase::getNbRegime)
+    .def("getNbControl", &StOpt::OptimizerDPCutBase::getNbControl)
+    .def("getSimulator", &StOpt::OptimizerDPCutBase::getSimulator)
+    .def("getSimuFuncSize", &StOpt::OptimizerDPCutBase::getSimuFuncSize)
+    ;
+
     // map optimizer DP base for tree
     //******************************
     py::class_<StOpt::OptimizerDPTreeBase, std::shared_ptr<StOpt::OptimizerDPTreeBase>, PyOptimizerDPTreeBase>(m, "OptimizerDPTreeBase")
@@ -1165,15 +1385,28 @@ PYBIND11_MODULE(StOptGlobal, m)
          const   std::shared_ptr< StOpt::SpaceGrid > &, const  std::shared_ptr< StOpt::OptimizerDPBase > & >())
     .def("oneStep", &PySimulateStepRegression::oneStepWrap)
     ;
-
-
     py::class_<PySimulateStepRegressionControl, std::shared_ptr<PySimulateStepRegressionControl> >(m, "SimulateStepRegressionControl")
     .def(py::init< BinaryFileArchiveStOpt &,  const int &,  const std::string &,
          const   std::shared_ptr< StOpt::SpaceGrid > &, const  std::shared_ptr< StOpt::OptimizerDPBase > & >())
     .def("oneStep", & PySimulateStepRegressionControl::oneStepWrap)
     ;
+    
+    /// map transition step with cuts
+    py::class_<PyTransitionStepRegressionDPCut, std::shared_ptr<PyTransitionStepRegressionDPCut> >(m, "TransitionStepRegressionDPCut")
+    .def(py::init< const std::shared_ptr<StOpt::SpaceGrid> &,
+         const  std::shared_ptr<StOpt::SpaceGrid> &, const std::shared_ptr<StOpt::OptimizerDPCutBase > &>())
+    .def("oneStep", & PyTransitionStepRegressionDPCut::oneStepWrap)
+    .def("dumpContinuationCutsValues", & PyTransitionStepRegressionDPCut::dumpContinuationCutsValuesWrap)
+    .def("dumpBellmanCutsValues", & PyTransitionStepRegressionDPCut::dumpBellmanCutsValuesWrap)
+    ;
 
-
+    /// associated cut simulation step
+    py::class_<PySimulateStepRegressionCut, std::shared_ptr<PySimulateStepRegressionCut> >(m, "SimulateStepRegressionCut")
+    .def(py::init< BinaryFileArchiveStOpt &,  const int &,  const std::string &,
+         const   std::shared_ptr< StOpt::SpaceGrid > &, const  std::shared_ptr< StOpt::OptimizerDPCutBase > & >())
+    .def("oneStep", &PySimulateStepRegressionCut::oneStepWrap)
+    ;
+    
     /// map one time step transition by tree
     py::class_<PyTransitionStepTreeDP, std::shared_ptr<PyTransitionStepTreeDP> >(m, "TransitionStepTreeDP")
     .def(py::init< const std::shared_ptr<StOpt::SpaceGrid> &,
@@ -1274,7 +1507,23 @@ PYBIND11_MODULE(StOptGlobal, m)
     .def("oneStep", &PyTransitionStepRegressionSwitchDist::oneStepWrap)
     .def("dumpContinuationValues", &PyTransitionStepRegressionSwitchDist::dumpContinuationValuesWrap)
     ;
+    
+    /// map transition step with cuts
+    py::class_<PyTransitionStepRegressionDPCutDist, std::shared_ptr<PyTransitionStepRegressionDPCutDist> >(m, "TransitionStepRegressionDPCutDist")
+    .def(py::init< const std::shared_ptr<StOpt::SpaceGrid> &,
+         const  std::shared_ptr<StOpt::SpaceGrid> &, const std::shared_ptr<StOpt::OptimizerDPCutBase > &>())
+    .def("oneStep", & PyTransitionStepRegressionDPCutDist::oneStepWrap)
+    .def("dumpContinuationCutsValues", & PyTransitionStepRegressionDPCutDist::dumpContinuationCutsValuesWrap)
+    .def("dumpBellmanCutsValues", & PyTransitionStepRegressionDPCutDist::dumpBellmanCutsValuesWrap)
+    ;
 
+    /// associated cut simulation step
+    py::class_<PySimulateStepRegressionCutDist, std::shared_ptr<PySimulateStepRegressionCutDist> >(m, "SimulateStepRegressionCutDist")
+    .def(py::init< BinaryFileArchiveStOpt &,  const int &,  const std::string &,
+         const   std::shared_ptr< StOpt::SpaceGrid > &, const  std::shared_ptr< StOpt::OptimizerDPCutBase > & , const bool &>())
+    .def("oneStep", &PySimulateStepRegressionCutDist::oneStepWrap)
+    ;
+    
     m.def("reconstructProc0Mpi", & pyReconstructProc0Mpi);
     m.def("reconstructProc0ForIntMpi", & pyReconstructProc0ForIntMpi);
 
