@@ -35,6 +35,8 @@
 #include "StOpt/core/grids/RegularSpaceIntGrid.h"
 #include "StOpt/core/grids/OneDimRegularSpaceGrid.h"
 #include "StOpt/core/grids/OneDimSpaceGrid.h"
+#include "StOpt/core/grids/Mesh2D.h"
+#include "StOpt/core/grids/GridAdapt2D.h"
 #include "StOpt/core/sparse/sparseGridUtils.h"
 #include  "StOpt/python/Pybind11VectorAndList.h"
 
@@ -1116,4 +1118,12 @@ PYBIND11_MODULE(StOptGrids, m)
             .def("getMesh", &StOpt::OneDimSpaceGrid::getMesh)
             .def("getNbStep", &StOpt::OneDimSpaceGrid::getNbStep)
             ;
+    py::class_<StOpt::Mesh2D, std::shared_ptr<StOpt::Mesh2D>>(m, "Mesh2D")
+            .def(py::init<const double &,    const double &, const  double &,   const double &,   const int &,  const int &,  const int &,   const int &>())
+            .def("split", &StOpt::Mesh2D::split)
+            ;
+
+    py::class_<StOpt::GridAdapt2D, std::shared_ptr<StOpt::GridAdapt2D>>(m, "GridAdapt2D")
+            .def(py::init<const double &, const double &, const int &, const double &, const double &, const int & >())
+            ;
 }
