@@ -17,6 +17,7 @@
 #include <pybind11/stl.h>
 #include "StOpt/core/grids/SpaceGrid.h"
 #include "StOpt/core/grids/FullGrid.h"
+#include "StOpt/core/grids/GridAdapt2D.h"
 #include "StOpt/core/utils/Polynomials1D.h"
 #include "StOpt/core/utils/version.h"
 #include "StOpt/regression/LocalLinearRegression.h"
@@ -36,7 +37,9 @@
 #include "StOpt/regression/ContinuationValue.h"
 #include "StOpt/regression/ContinuationValueDet.h"
 #include "StOpt/regression/ContinuationCuts.h"
+#include "StOpt/regression/ContinuationCutsGridAdaptNonConcave.h"
 #include "StOpt/regression/GridAndRegressedValueGeners.h"
+#include "StOpt/python/ContinuationCutsGridAdaptNonConcaveWrap.h"
 
 
 namespace py = pybind11;
@@ -121,6 +124,8 @@ public :
 };
 
 
+
+
 /// \brief Encapsulation for regression module
 PYBIND11_MODULE(StOptReg, m)
 {
@@ -339,4 +344,31 @@ PYBIND11_MODULE(StOptReg, m)
     .def("getNbSimul", &StOpt::ContinuationCuts::getNbSimul)
     .def("getParticles", &StOpt::ContinuationCuts::getParticles)
     ;
+
+    // map continuation cut for grid adaptation
+    py::class_<StOpt::ContinuationCutsGridAdaptNonConcaveWrap, std::shared_ptr<StOpt::ContinuationCutsGridAdaptNonConcaveWrap> >(m, "ContinuationCutsGridAdaptNonConcave")
+    .def(py::init <  const  std::shared_ptr< StOpt::GridAdapt2D >   &, const  std::shared_ptr< StOpt::BaseRegression >   &, const  Eigen::ArrayXXd &>())
+    .def(py::init < const  std::shared_ptr< StOpt::BaseRegression >   &>())
+    .def("loadForSimulation", [ ](StOpt::ContinuationCutsGridAdaptNonConcave &  p_x, const  std::shared_ptr< StOpt::GridAdaptBase > &p_grid,
+                                  const std::shared_ptr< StOpt::BaseRegression >   &p_condExp, const py::list & ns)
+    {
+        Eigen::Array< Eigen::ArrayXXd, Eigen::Dynamic, 1  >  values(ns.size());
+        for (size_t i = 0; i < ns.size(); ++i)
+        {
+            values(i) = ns[i].cast<Eigen::ArrayXXd>();
+        }
+        return p_x.loadForSimulation(p_grid, p_condExp, values);
+        ;
+    }
+        )
+    .def("getGrid", &StOpt::ContinuationCutsGridAdaptNonConcave::getGrid)
+    .def("getCondExp", &StOpt::ContinuationCutsGridAdaptNonConcave::getCondExp)
+    .def("getNbSimul", &StOpt::ContinuationCutsGridAdaptNonConcave::getNbSimul)
+    .def("getParticles", &StOpt::ContinuationCutsGridAdaptNonConcave::getParticles)
+    .def("getMeshNotConcASim", &StOpt::ContinuationCutsGridAdaptNonConcave::getMeshNotConcASim)
+    .def("getCutsASimByCoord", &StOpt::ContinuationCutsGridAdaptNonConcaveWrap::getCutsASimByCoord)
+    .def("getCutsASimByNumber", &StOpt::ContinuationCutsGridAdaptNonConcaveWrap::getCutsASimByNumber)
+    .def(" getCutsConcGatherASimByCoord", &StOpt::ContinuationCutsGridAdaptNonConcaveWrap::getCutsConcGatherASimByCoord)
+    .def(" getCutsConcGatherASimByNumber", &StOpt::ContinuationCutsGridAdaptNonConcaveWrap::getCutsConcGatherASimByNumber)
+    ;
 }
