@@ -112,8 +112,9 @@ void testGasStorageFullGridTree(shared_ptr< FullGrid > &p_grid, const double &p_
 
 
     // create archive
-    string nameTree = "Tree";
+    string nameTree = "TreeGasStorage";
 #ifdef USE_MPI
+    nameTree += to_string(world.size());
     if (world.rank() == 0)
 #endif
         tree.dump(nameTree, indexT);
