@@ -3,6 +3,9 @@
 #include <fstream>
 #include <memory>
 #include <functional>
+#ifdef USE_MPI
+#include <boost/mpi.hpp>
+#endif
 #include <Eigen/Dense>
 #include "StOpt/core/grids/SpaceGrid.h"
 #include "StOpt/dp/OptimizerDPBase.h"
@@ -20,11 +23,16 @@
 /// \param p_regressor         regressor object
 /// \param p_funcFinalValue    function defining the final value
 /// \param p_fileToDump        file to dump continuation values
+/// \param p_world              MPI communicator
 ///
 void  DpTimeNonEmissive(const std::shared_ptr<StOpt::FullGrid> &p_grid,
                         const std::shared_ptr<StOpt::OptimizerDPBase > &p_optimize,
                         const std::shared_ptr<StOpt::BaseRegression> &p_regressor,
                         const std::function<double(const int &, const Eigen::ArrayXd &, const Eigen::ArrayXd &)>   &p_funcFinalValue,
-                        const std::string   &p_fileToDump);
+                        const std::string   &p_fileToDump
+#ifdef USE_MPI
+                        , const boost::mpi::communicator &p_world
+#endif
+                       );
 
 #endif /* DPTIMENONEMISSIVE_H */
