@@ -329,11 +329,11 @@ void  TestLocalDiscrLastDimRegressionSerialization(const int &p_nDim,  const int
 
     // archive
     {
-        BinaryFileArchive ar("archive", "w");
+        BinaryFileArchive ar("archiveLDL", "w");
         ar << Record(*localRegressor, "Regressor", "Top") ;
     }
     {
-        BinaryFileArchive ar("archive", "r");
+        BinaryFileArchive ar("archiveLDL", "r");
         Regressor reg;
         Reference< Regressor> (ar, "Regressor", "Top").restore(0, &reg);
         for (int is = 0; is < x.cols(); ++is)
@@ -424,13 +424,13 @@ void TestContinuationValueDiscr(const int &p_nDim,  const int &p_nbDiscr, const
             BOOST_CHECK_CLOSE(regressedByContinuation(is), regressedValues(sizeForStock / 2, is), accuracyEqual);
 
         // default non compression
-        BinaryFileArchive ar("archive", "w");
+        BinaryFileArchive ar("archiveLDL1", "w");
         ar << Record(continuation, "FirstContinuation", "Top") ;
 
     }
     {
         // read archive
-        BinaryFileArchive ar("archive", "r");
+        BinaryFileArchive ar("archiveLDL1", "r");
         ContinuationValue contRead;
         Reference< ContinuationValue >(ar, "FirstContinuation", "Top").restore(0, &contRead);
         ArrayXd ptStock(1) ;
