@@ -19,6 +19,7 @@
 #include "StOpt/regression/LocalLinearRegressionGeners.h"
 #include "StOpt/regression/ContinuationValueGeners.h"
 #include "StOpt/regression/ContinuationCutsGeners.h"
+#include "StOpt/regression/ContinuationCutsGridAdaptNonConcaveGeners.h"
 
 using namespace std;
 using namespace Eigen;
@@ -74,6 +75,19 @@ public :
     }
 };
 
+class  Func3
+{
+
+public :
+
+    Func3() {}
+
+    double operator()(const double &p_x, const double &p_y) const
+    {
+        return  p_x * p_y + p_x + p_y + 1.;
+    }
+};
+
 // test for different  dimensions
 void testDimension(const bool &p_bRotate, const int &p_nDim, const int &p_nbSimul, const int &p_nMesh, const double &p_accuracyRegression)
 {
@@ -574,6 +588,175 @@ BOOST_AUTO_TEST_CASE(testContCut)
 }
 
 
+// test serialization for continuation cuts values for 2D adapted grids
+void TestContinuationCutsGridAdapt2D(const int &p_nDim, const int &p_nbSimul, const int &p_nbMesh)
+{
+#if defined   __linux
+    enable_abort_on_floating_point_exception();
+#endif
+    ArrayXi  nbMesh = ArrayXi::Constant(p_nDim, p_nbMesh);
+    // generator Mersene Twister
+    boost::mt19937 generator;
+    boost::normal_distribution<double> alea_n;
+    boost::variate_generator<boost::mt19937 &, boost::normal_distribution<double> > normal_random(generator, alea_n) ;
+    std::function< double(double) > f = Func1();
+    ArrayXXd x = ArrayXXd::Random(p_nDim, p_nbSimul);     // test archive
+    ArrayXXd  regressedValues;
+    ArrayXXd  hyperStock(2, 2) ;
+
+    // grid for storage
+    double xMin = 0;
+    double xMax = 5.;
+    int nbMeshX = 5;
+    double yMin = 0.;
+    double yMax = 4.;
+    int nbMeshY = 4 ;
+
+    // nb stock points
+    int sizeForStock  = (nbMeshX + 1) * (nbMeshY + 1);
+
+    // grid
+    shared_ptr< GridAdapt2D > grid2D = make_shared<GridAdapt2D>(xMin, xMax, nbMeshX, yMin, yMax, nbMeshY);
+
+    {
+        // second member to regress with one stock  (one storage, so one sensibility)
+        ArrayXXd toRegress(sizeForStock, p_nbSimul);
+        ArrayXXd toRegressCut(sizeForStock, p_nbSimul * 3);
+        for (int is  = 0;  is < p_nbSimul; ++is)
+        {
+            double prod = f(x(0, is));
+            for (int id = 1 ; id < p_nDim ; ++id)
+                prod *=  f(x(id, is));
+            double uncertainty = 4 * normal_random();
+            for (int j = 0; j < nbMeshY + 1; ++j)
+                for (int i = 0; i < nbMeshX + 1; ++i)
+                {
+                    int istock = i + j * (nbMeshX + 1);
+                    toRegress(istock, is) = prod * istock  +  uncertainty;
+                    toRegressCut(istock, is) = prod * istock +  uncertainty;
+                    toRegressCut(istock, is + p_nbSimul) = prod ; // sensibility equal to prod in x dimension
+                    toRegressCut(istock, is + 2 * p_nbSimul) = prod ; // sensibility equal to prod in y dimension
+                }
+
+        }
+        // conditional expectation
+        shared_ptr<LocalLinearRegression> localRegressor = make_shared<LocalLinearRegression>(false, x, nbMesh);
+
+        // regress  directly with regressor
+        regressedValues = localRegressor->getAllSimulationsMultiple(toRegress);
+
+        // creation continuation value object
+        ContinuationCutsGridAdaptNonConcave  contCut(grid2D, localRegressor,  toRegressCut.transpose());
+
+        // calculate all cuts
+        hyperStock(0, 0) = xMin;
+        hyperStock(0, 1) = xMax;
+        hyperStock(1, 0) = yMin;
+        hyperStock(1, 1) = yMax;
+
+        vector< pair<shared_ptr< GridAdaptBase>, shared_ptr<ArrayXXd>  > >   regressedCutsPMesh = contCut.getCutsAllSimulations(hyperStock);
+
+        // test only values
+        ArrayXi   pointNumber(4);
+        ArrayXd pointCoord(2);
+        for (const auto &pairMA : regressedCutsPMesh)
+        {
+            // for all points on the sub grid
+            std::vector< Eigen::ArrayXd > points = pairMA.first->getPoints();
+
+            for (size_t i = 0; i < points.size(); ++i)
+            {
+                pointCoord(0) = points[i][0];
+                pointCoord(1) = points[i][1];
+                // locate position in the initial grid
+                int ipos = static_cast<int>(pointCoord(0)) +  static_cast<int>(pointCoord(1)) * (nbMeshX + 1);
+                for (int isim = 0; isim < p_nbSimul; ++isim)
+                {
+                    // reconstruct value from cuts coefficients
+                    double valReconst = (*pairMA.second)(isim, i) + (*pairMA.second)(isim + p_nbSimul, i) * pointCoord(0) + (*pairMA.second)(isim + 2 * p_nbSimul, i) * pointCoord(1);
+                    BOOST_CHECK_CLOSE(valReconst, regressedValues(ipos, isim), accuracyEqual);
+                }
+            }
+        }
+        for (int isim = 0; isim < p_nbSimul;  ++isim)
+        {
+            vector< pair<shared_ptr< GridAdaptBase>, shared_ptr<ArrayXXd>  > >   regressedCutsPMesh = contCut.getCutsASim(hyperStock, isim);
+
+            for (const auto &pairMA : regressedCutsPMesh)
+            {
+                // for all points on the sub grid
+                std::vector< Eigen::ArrayXd > points = pairMA.first->getPoints();
+
+                for (size_t i = 0; i < points.size(); ++i)
+                {
+                    pointCoord(0) = points[i][0];
+                    pointCoord(1) = points[i][1];
+                    // locate position in the initial grid
+                    int ipos = static_cast<int>(pointCoord(0)) +  static_cast<int>(pointCoord(1)) * (nbMeshX + 1);
+                    // reconstruct value from cuts coefficients
+                    double valReconst = (*pairMA.second)(0, i) + (*pairMA.second)(1, i) * pointCoord(0) + (*pairMA.second)(2, i) * pointCoord(1);
+                    BOOST_CHECK_CLOSE(valReconst, regressedValues(ipos, isim), accuracyEqual);
+                }
+            }
+        }
+
+        // default non compression
+        BinaryFileArchive ar("archiveLLR2", "w");
+        ar << Record(contCut, "FirstContinuation", "Top") ;
+
+    }
+    {
+        // read archive
+        BinaryFileArchive ar("archiveLLR2", "r");
+        ContinuationCutsGridAdaptNonConcave contCut;
+        Reference< ContinuationCutsGridAdaptNonConcave >(ar, "FirstContinuation", "Top").restore(0, &contCut);
+
+        // calculate all cuts
+        hyperStock(0, 0) = xMin;
+        hyperStock(0, 1) = xMax;
+        hyperStock(1, 0) = yMin;
+        hyperStock(1, 1) = yMax;
+
+        // test only values
+        ArrayXi   pointNumber(4);
+        ArrayXd pointCoord(2);
+        // point test
+        ArrayXd pointCoordTest(2);
+        ArrayXd regPoint = ArrayXd::Constant(p_nDim, 0.5) ;
+        for (int isim = 0; isim < p_nbSimul;  ++isim)
+        {
+            vector< pair<shared_ptr< GridAdaptBase>, shared_ptr<ArrayXXd>  > >   regressedCutsPMesh = contCut.getCutsASim(hyperStock, x.col(isim));
+
+            for (const auto &pairMA : regressedCutsPMesh)
+            {
+                // for all points on the sub grid
+                std::vector< Eigen::ArrayXd > points = pairMA.first->getPoints();
+
+                for (size_t i = 0; i < points.size(); ++i)
+                {
+                    pointCoord(0) = points[i][0];
+                    pointCoord(1) = points[i][1];
+                    // reconstruct value from cuts coefficients : just to test
+                    double valReconst = (*pairMA.second)(0, i) + (*pairMA.second)(1, i) * pointCoord(0) + (*pairMA.second)(2, i) * pointCoord(1);
+                    BOOST_CHECK(abs(valReconst) < 1e10);
+                }
+            }
+        }
+    }
+}
+
+
+
+BOOST_AUTO_TEST_CASE(testContCutGridAdapt2D)
+{
+#if defined   __linux
+    enable_abort_on_floating_point_exception();
+#endif
+
+    TestContinuationCutsGridAdapt2D(1, 10, 1);
+}
+
+
 BOOST_AUTO_TEST_CASE(testCloneLocal)
 {
 #if defined   __linux
