@@ -322,11 +322,11 @@ void  TestLocalRegressionSerialization(const int &p_nDim, const int &p_nbSimul,
         }
 
         // archive
-        BinaryFileArchive ar("archive", "w");
+        BinaryFileArchive ar("archiveLSSL", "w");
         ar << Record(*localRegressor, "Regressor", "Top") ;
     }
     {
-        BinaryFileArchive ar("archive", "r");
+        BinaryFileArchive ar("archiveLSSL", "r");
         LocalSameSizeLinearRegression reg;
         Reference< LocalSameSizeLinearRegression> (ar, "Regressor", "Top").restore(0, &reg);
         for (int is = 0; is < x.size(); ++is)
@@ -402,13 +402,13 @@ void TestContinuationValue(const int &p_nDim, const int &p_nbSimul, const int &p
             BOOST_CHECK_CLOSE(regressedByContinuation(is), regressedValues(sizeForStock / 2, is), accuracyEqual);
 
         // default non compression
-        BinaryFileArchive ar("archive", "w");
+        BinaryFileArchive ar("archiveLSSL1", "w");
         ar << Record(continuation, "FirstContinuation", "Top") ;
 
     }
     {
         // read archive
-        BinaryFileArchive ar("archive", "r");
+        BinaryFileArchive ar("archiveLSSL1", "r");
         ContinuationValue contRead;
         Reference< ContinuationValue >(ar, "FirstContinuation", "Top").restore(0, &contRead);
         ArrayXd ptStock(1) ;
