@@ -2,6 +2,7 @@
 # All Rights Reserved
 # This code is published under the GNU Lesser General Public License (GNU LGPL)
 import math
+import numpy as np
 
 # Defines a specialization of the RegularSpaceGrid  object in one dimension
 
@@ -32,7 +33,7 @@ class OneDimRegularSpaceGrid:
         if (self.m_lowValue <= p_coord) == False:
             pass
         
-        return min(int(math.ceil((p_coord - self.m_lowValue) / self.m_step)), self.m_nbStep)
+        return np.floor((p_coord - self.m_lowValue) / self.m_step).astype(int)
     
     # get back value
     
